from keras.models import Model
from keras.optimizers import Adam
from keras.layers import Input, Conv2D, MaxPooling2D, UpSampling2D
from keras.utils import multi_gpu_model


def AutoEncoder(gpu_num=1):
    input_layer = Input(shape=(128, 128, 3))

    conv11 = Conv2D(32, (3, 3), activation='relu', padding='same')(input_layer)
    conv12 = Conv2D(32, (3, 3), activation='relu', padding='same')(conv11)
    pool1 = MaxPooling2D()(conv12)

    conv21 = Conv2D(16, (3, 3), activation='relu', padding='same')(pool1)
    conv22 = Conv2D(16, (3, 3), activation='relu', padding='same')(conv21)
    encoded = MaxPooling2D()(conv22)

    conv31 = Conv2D(8, (3, 3), activation='relu', padding='same')(encoded)
    conv32 = Conv2D(8, (3, 3), activation='relu', padding='same')(conv31)

    up1 = UpSampling2D()(conv32)
    conv41 = Conv2D(16, (3, 3), activation='relu', padding='same')(up1)
    conv42 = Conv2D(16, (3, 3), activation='relu', padding='same')(conv41)

    up2 = UpSampling2D()(conv42)
    conv51 = Conv2D(32, (3, 3), activation='relu', padding='same')(up2)
    conv52 = Conv2D(32, (3, 3), activation='relu', padding='same')(conv51)

    decoded = Conv2D(3, (1, 1), activation='sigmoid', padding='same')(conv52)

    model = Model(inputs=input_layer, outputs=decoded)

    if gpu_num >= 2:
        model = multi_gpu_model(model, gpus=gpu_num)

    adam = Adam(lr=0.0001)
    model.compile(optimizer=adam, loss='binary_crossentropy')

    return model
